# Configuration
## oidc-agent Directory
An oidc-agent directory will be created when using oidc-gen for the first time. 
Depending on the existence of ```~/.config``` it will be located at one of these
locations:
- ```~/.config/oidc-agent```
- ```~/.oidc-agent```

Alternatively the location can also be set through an environment variable
called ```OIDC_CONF_DIR```. However, we note that this environment variable has
to be present whenever you use one of the ```oidc-``` binaries. It is therefore
recommend to set it in the ```.bashrc``` or similar. See also [Persisting
oidc-agent](#persisting-oidc-agent-through-bashrc).

All account configuration generated by ```oidc-gen``` are saved in this
oidc-agent directory. Additionally there is a config file named ```issuer.config```. This file can be used to specify a list of issuers (one issuer per line) that are used as suggestions by oidc-gen. oidc-gen will also update this issuer list after an account configuration was created successfully. oidc-agent installs a similar file under ```/etc/oidc-agent```, however, that file should not be edited by the user, but it might be updated with new oidc-agent versions.

### Default Account Configuration for a Provider
The just mentioned ```issuer.config``` file in the oidc-agent directory can also
be used to set an default account configuration file for each provider by adding
the shortname of this account configuration after the issuer url.
A line in the ```issuer.config``` file should look the following:
```
<issuer_url>[<space><shortname>]
```

## oidc-agent Integration
### Xsession Integration
oidc-agent has support for integration with Xsession, which is enabled by
default. This means oidc-agent will automatically start iat the beginning of an 
Xsession and then be available throughout that session (i.e. you can connect to
the agent from every terminal).

To disable / reenable this behavior edit the ```/etc/X11/Xsession.options``` file. If the line ```use-oidc-agent``` is present oidc-agent will automatically started at the beginning of an Xsession.

To pass command line options to the automatically started agent edit the
```OIDCAGENTARGS``` variable in the file ```/etc/X11/Xsession.d/91oidc-agent```.

### Persisting oidc-agent Through ```.bashrc```
A simple alternative to Xsession integration is to include this line in your
`.bashrc`:
```
test -e ~/tmp/oidc-agent.env && . ~/tmp/oidc-agent.env
```
And to run the agent as `oidc-agent > ~/tmp/oidc-agent.env` (you might want to
define an alias for that).
From now on every new shell should have access to the agent. 

You can test this with:
```
oidc-token <shortname>
```

## Agent Forwarding
When using ```ssh``` to connect to a remote server there might be the use case
where the user or an application on the remote server wants to receive an access token from
the local agent. This is possible by forwarding the UNIX domain socket used for
communicating with the agent. This can be done using the ```-R``` option of
```ssh```. Example:
```
ssh -R /tmp/oidc-forward:$OIDC_SOCK  user@host
```
However, if you do this you still have to manually export the ```OIDC_SOCK```
envenvironment variable on the server (```export
OIDC_SOCK=/tmp/oidc-forward```).

**To use agent forwarding we recommend the following configurations:**
Put the following in your ```.bashrc``` on the server:
```
test -z $OIDC_SOCK && {
    export OIDC_SOCK=`ls -rt /tmp/oidc-forward-* | tail -n 1`
}
```
And in the ```.bash_aliases``` on your local machine:
```
alias OA='echo -R /tmp/oidc-forward-$RANDOM:$OIDC_SOCK'
```
You can then call ssh the following way:
```
ssh user@host `OA`
```

## Other Configuration
Generally oidc-agent does not use any configuration files. (Configuration files
for [seccomp](security.md#seccomp) and the already mentioned ```issuer.conf``` excluded.) 
Configuration needed is done mostly through command line options to the
different components and in some cases through environement variables.
If some command line options are used for every call, it makes sense to define
an alias for it in ```.bashrc``` or ```.bash_aliases```, e.g. ```alias oidc-add="oidc-add --pw-store=3600"```.
